# -*- coding: utf-8 -*-

from zope.component import getGlobalSiteManager
from zope.interface import implementedBy, providedBy


def queryClassMultiAdapter(adapts, ob, interface, name=u''):
    """This function searches the component registry for any adapter
    registered for an instance of the given class. It lookups and returns
    it correctly factored.
    """
    sm = getGlobalSiteManager()
    klass = adapts[0]
    required = implementedBy(klass)
    lookfor = (required,) + tuple(providedBy(a) for a in adapts[1:])
    factory = sm.adapters.lookup(lookfor, interface, name)
    if factory is not None:
        return factory(ob, *adapts[1:])
    return None



#!/usr/bin/python
# -*- coding: utf-8 -*-

from zope.i18nmessageid import MessageFactory
i18n = MessageFactory("dolmen.forms.crud")

from dolmen.forms.crud.interfaces import (
    IAdding, IFactoryAdding, IFieldsCustomization)
from dolmen.forms.crud.addview import Adder
from dolmen.forms.crud.crudforms import (
    ApplicationForm, Display, Add, Edit, Delete)
from dolmen.forms.crud.customize import FieldsCustomizer



#!/usr/bin/python
# -*- coding: utf-8 -*-

from dolmen.forms.base.utils import apply_data_event
from zeam.form import base
from zeam.form.base.markers import SUCCESS, FAILURE
from zope.event import notify
from zope.i18nmessageid import MessageFactory
from zope.lifecycleevent import ObjectCreatedEvent

_ = MessageFactory("dolmen.forms.crud")


class Add(base.Action):
    """Add action for an IAdding context.
    """

    def __init__(self, title, factory):
        super(Add, self).__init__(title)
        self.factory = factory

    def __call__(self, form):
        data, errors = form.extractData()
        if errors:
            return FAILURE

        obj = self.factory(**data)
        notify(ObjectCreatedEvent(obj))
        form.context.add(obj)

        form.flash(_(u"Content created"))
        form.redirect(form.url(obj))

        return SUCCESS


class Update(base.Action):
    """Update action for any locatable object.
    """

    def __call__(self, form):
        data, errors = form.extractData()
        if errors:
            return FAILURE

        apply_data_event(form.fields, form.context, data)

        form.flash(_(u"Content updated"))
        form.redirect(form.url(form.context))

        return SUCCESS


class Delete(base.Action):
    """Delete action for any locatable context.
    """
    successMessage = _(u"The object has been deleted.")
    failureMessage = _(u"This object could not be deleted.")
    
    def __call__(self, form):
        container = form.context.__parent__
        name = form.context.__name__

        if name in container:
            try:
                del container[name]
                form.status = self.successMessage
                form.flash(form.status)
                form.redirect(form.url(container))
                return SUCCESS
            except ValueError:
                pass

        form.status = self.failureMessage
        form.flash(form.status)
        form.redirect(form.url(form.context))
        return FAILURE



# -*- coding: utf-8 -*-

import zope.i18n
import dolmen.content as content
import zeam.form.ztk as form
import grokcore.component as grok

from dolmen.forms.base import ApplicationForm
from dolmen.forms.crud import actions as formactions
from dolmen.forms.crud.interfaces import IFactoryAdding, IFieldsCustomization
from dolmen.forms.crud.utils import queryClassMultiAdapter

from zeam.form.base import Fields, Actions
from zope.cachedescriptors.property import CachedProperty
from zope.component import queryMultiAdapter
from zope.i18nmessageid import MessageFactory

_ = MessageFactory("dolmen.forms.crud")


class Add(ApplicationForm):
    """The add form itself is not protected. The security is checked on
    'update'. It checks if the 'require' directive of the factored item
    is respected on the context.
    """
    grok.baseclass()
    grok.title(_(u"Add"))
    grok.name('dolmen.add')
    grok.context(IFactoryAdding)

    @property
    def label(self):
        return zope.i18n.translate(self.context.factory.title,
                                   context=self.request)

    @CachedProperty
    def fields(self):
        ifaces = self.context.factory.getSchema()
        fields = Fields(*ifaces).omit('__parent__')

        modifier = queryClassMultiAdapter(
            (self.context.factory.factory, self, self.request),
            self.context, IFieldsCustomization)

        if modifier is not None:
            return modifier(fields)
        return fields

    @CachedProperty
    def actions(self):
        add = formactions.Add(_("Add"), self.context.factory)
        return Actions(add)


class Edit(ApplicationForm):
    grok.baseclass()
    grok.name('edit')
    grok.title(_(u"Edit"))
    grok.context(content.IBaseContent)

    ignoreContent = False
    ignoreRequest = False
    actions = Actions(formactions.Update(_("Update")))

    @property
    def label(self):
        label = _(u"edit_action", default=u"Edit: $name",
                 mapping={"name": self.context.title})
        return zope.i18n.translate(label, context=self.request)

    @CachedProperty
    def fields(self):
        iface = content.schema.bind().get(self.context)
        fields = Fields(*iface).omit('__parent__')
        modifier = queryMultiAdapter(
            (self.context, self, self.request), IFieldsCustomization)

        if modifier is not None:
            return modifier(fields)
        return fields


class Display(ApplicationForm):
    grok.baseclass()
    grok.title(_(u"View"))
    grok.context(content.IBaseContent)

    mode = "display"
    ignoreRequest = True
    ignoreContent = False

    @property
    def label(self):
        return self.context.title

    @CachedProperty
    def fields(self):
        iface = content.schema.bind().get(self.context)
        fields = form.Fields(*iface).omit('__parent__', 'title')
        modifier = queryMultiAdapter(
            (self.context, self, self.request), IFieldsCustomization)

        if modifier is not None:
            return modifier(fields)
        return fields


class Delete(ApplicationForm):
    """A confirmation for to delete an object.
    """
    grok.baseclass()
    grok.title(_(u"Delete"))
    grok.context(content.IBaseContent)

    label = _(u"Delete")
    description = _(u"Are you really sure ?")
    actions = Actions(formactions.Delete(_("Delete")))



# -*- coding: utf-8 -*-

import dolmen.content
from zope.schema import Object
from zope.location.interfaces import ILocation
from zope.interface import Interface, Attribute


class IAdding(ILocation):
    """Defines an abstraction layer for the adding mechanism.
    Historically, the interface was located in the `zope.app.container`.
    """
    context = Attribute("The context of the adding object")
    request = Attribute("The HTTP request")


class IFactoryAdding(IAdding):
    """An IFactoryAdding extends an IAdding by adding the notion of Factory.
    See `dolmen.content.IFactory` definition for more information.
    """
    factory = Object(
        missing_value=None,
        title=u"The factory generating the content.",
        schema=dolmen.content.IFactory)


class IFieldsCustomization(Interface):
    """Defines a form customization. A form customization is an adapter
    that allows to modify the fields of a form and their rendering.
    """
    context = Attribute("The context of the customized form")
    request = Attribute("The HTTP request")
    form = Attribute("The form object")

    def __call__(fields):
        """Must returns an instance of z3c.form.Fields or a modified
        versions of the original fields arg.
        """
